==> fred-backend-registry_2.10.1.orig.tar.gz: fred-backend-registry-2.10.1/src/util/sql_exec_in_thread.cc <==
/*
 * Copyright (C) 2018-2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */
#include "src/util/sql_exec_in_thread.hh"

#include "src/util/measure_duration.hh"

#include "util/log/context.hh"
#include "util/log/logger.hh"

#include <boost/variant.hpp>

#include <atomic>
#include <chrono>
#include <thread>

namespace Fred {
namespace Registry {
namespace Util {

namespace {

void activate_logging_context()
{
    static std::atomic<std::size_t> cnt{0};
    thread_local Logging::Context ctx("sql-in-thread-solver<" + std::to_string(cnt.fetch_add(1, std::memory_order_relaxed)) + ">");
}

template <typename T>
Database::Result sql_exec_out_of_transaction(T callable)
{
    const auto conn = Database::get_default_manager<Database::StandaloneConnectionFactory>().acquire();
    const struct LogDuration : MeasureDuration<LogDuration>
    {
        static void measure_completed(typename MeasureDuration<LogDuration>::Time duration_ms)
        {
            LOGGER.debug("exec done in " + std::to_string(duration_ms) + "ms");
        }
    } log_duration;

    return callable(*conn);
}

}//namespace Fred::Registry::Util::{anonymous}

SqlInThreadSolver sql_exec_in_thread(const std::string& sql)
{
    return SqlInThreadSolver(sql);
}

SqlInThreadSolver sql_exec_in_thread(const std::string& sql, const Database::QueryParams& params)
{
    return SqlInThreadSolver(sql, params);
}

Database::Result sql_exec(const std::string& sql)
{
    return sql_exec_out_of_transaction([&](Database::StandaloneConnection& conn) { return conn.exec(sql); });
}

Database::Result sql_exec(const std::string& sql, const Database::QueryParams& params)
{
    return sql_exec_out_of_transaction([&](Database::StandaloneConnection& conn) { return conn.exec_params(sql, params); });
}

class SqlInThreadSolver::Solver
{
public:
    Solver(const std::string& sql)
        : duration_ms_{},
          thread_{[this, sql]() { activate_logging_context(); this->exec(sql); }}
    { }
    Solver(const std::string& sql, const Database::QueryParams& params)
        : duration_ms_{},
          thread_{[this, sql, params]() { activate_logging_context(); this->exec(sql, params); }}
    { }
    const Database::Result& join()
    {
        thread_.join();
        GetDatabaseResult visitor;
        return result_.apply_visitor(visitor);
    }
    void wait() noexcept
    {
        if (thread_.joinable())
        {
            thread_.join();
        }
    }
    int get_duration_in_ms() const
    {
        return duration_ms_;
    }
private:
    template <typename T>
    void solve(T callable)
    {
        auto t0 = std::chrono::steady_clock().now();
        try
        {
            const auto conn = Database::get_default_manager<Database::StandaloneConnectionFactory>().acquire();
            t0 = std::chrono::steady_clock().now();
            result_ = callable(*conn);
        }
        catch (const Database::ResultFailed& e)
        {
            result_ = e;
        }
        catch (const Database::Exception& e)
        {
            result_ = e;
        }
        catch (const std::runtime_error& e)
        {
            result_ = e;
        }
        catch (const std::exception& e)
        {
            result_ = std::runtime_error(e.what());
        }
        catch (...)
        {
            result_ = std::runtime_error("unexpected exception caught");
        }
        const auto t1 = std::chrono::steady_clock().now();
        duration_ms_ = std::chrono::duration_cast<std::chrono::milliseconds>(t1 - t0).count();
        LOGGER.debug("exec done in " + std::to_string(duration_ms_) + "ms");
    }
    void exec(const std::string& sql)
    {
        this->solve([&](Database::StandaloneConnection& conn) { return conn.exec(sql); });
    }
    void exec(const std::string& sql, const Database::QueryParams& params)
    {
        this->solve([&](Database::StandaloneConnection& conn) { return conn.exec_params(sql, params); });
    }
    using ExecResult = boost::variant<Database::Result,
                                      Database::ResultFailed,
                                      Database::Exception,
                                      std::runtime_error>;
    struct GetDatabaseResult : boost::static_visitor<const Database::Result&>
    {
        const Database::Result& operator()(const Database::Result& result) const
        {
            return result;
        }
        template <typename E>
        const Database::Result& operator()(const E& e) const
        {
            throw e;
        }
    };
    ExecResult result_;
    int duration_ms_;
    std::thread thread_;
};

SqlInThreadSolver::SqlInThreadSolver()
    : running_task_()
{
}

SqlInThreadSolver::SqlInThreadSolver(SqlInThreadSolver&& src)
    : running_task_(std::move(src.running_task_))
{
}

SqlInThreadSolver::SqlInThreadSolver(const std::string& sql)
    : running_task_(std::make_unique<Solver>(sql))
{
}

SqlInThreadSolver::SqlInThreadSolver(const std::string& sql, const Database::QueryParams& params)
    : running_task_(std::make_unique<Solver>(sql, params))
{
}

SqlInThreadSolver::~SqlInThreadSolver()
{
    if (running_task_ != nullptr)
    {
        running_task_->wait();
    }
}

SqlInThreadSolver& SqlInThreadSolver::operator=(SqlInThreadSolver&& src)
{
    std::swap(running_task_, src.running_task_);
    return *this;
}

bool SqlInThreadSolver::is_joinable() const noexcept
{
    return running_task_ != nullptr;
}

SqlInThreadSolver::Result::Result()
    : result_(),
      duration_ms_()
{
}

SqlInThreadSolver::Result::Result(const Result& src)
    : result_(src.result_),
      duration_ms_(src.duration_ms_)
{
}

SqlInThreadSolver::Result::Result(SqlInThreadSolver&& solver)
    : result_(solver.running_task_->join()),
      duration_ms_(solver.running_task_->get_duration_in_ms())
{
    solver.running_task_.reset(nullptr);
}

SqlInThreadSolver::Result::~Result()
{
}

SqlInThreadSolver::Result& SqlInThreadSolver::Result::operator=(const Result& src)
{
    result_ = src.result_;
    duration_ms_ = src.duration_ms_;
    return *this;
}

SqlInThreadSolver::Result& SqlInThreadSolver::Result::operator=(SqlInThreadSolver&& solver)
{
    result_ = solver.running_task_->join();
    duration_ms_ = solver.running_task_->get_duration_in_ms();
    solver.running_task_.reset(nullptr);
    return *this;
}

const Database::Result& SqlInThreadSolver::Result::get_result() const
{
    return result_;
}

int SqlInThreadSolver::Result::get_duration_in_ms() const
{
    return duration_ms_;
}

}//namespace Fred::Registry::Util
}//namespace Fred::Registry
}//namespace Fred

==> fred-backend-registry_2.12.1.orig.tar.gz: fred-backend-registry-2.12.1/src/util/sql_exec_in_thread.cc <==
/*
 * Copyright (C) 2018-2022  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/util/sql_exec_in_thread.hh"

#include "src/util/measure_duration.hh"

#include "util/log/log.hh"

#include <boost/variant.hpp>

#include <atomic>
#include <chrono>
#include <thread>

namespace Fred {
namespace Registry {
namespace Util {

namespace {

template <typename T>
Database::Result sql_exec_out_of_transaction(T callable)
{
    const auto conn = Database::get_default_manager<Database::StandaloneConnectionFactory>().acquire();
    const struct LogDuration : MeasureDuration<LogDuration>
    {
        static void measure_completed(typename MeasureDuration<LogDuration>::Time duration_ms)
        {
            FREDLOG_DEBUG("exec done in " + std::to_string(duration_ms) + "ms");
        }
    } log_duration;

    return callable(*conn);
}

}//namespace Fred::Registry::Util::{anonymous}

SqlInThreadSolver sql_exec_in_thread(const std::string& sql)
{
    return SqlInThreadSolver(sql);
}

SqlInThreadSolver sql_exec_in_thread(const std::string& sql, const Database::QueryParams& params)
{
    return SqlInThreadSolver(sql, params);
}

Database::Result sql_exec(const std::string& sql)
{
    return sql_exec_out_of_transaction([&](Database::StandaloneConnection& conn) { return conn.exec(sql); });
}

Database::Result sql_exec(const std::string& sql, const Database::QueryParams& params)
{
    return sql_exec_out_of_transaction([&](Database::StandaloneConnection& conn) { return conn.exec_params(sql, params); });
}

class SqlInThreadSolver::Solver
{
public:
    Solver(const std::string& sql)
        : duration_ms_{},
          thread_{[this, sql]() { this->exec(sql); }}
    { }
    Solver(const std::string& sql, const Database::QueryParams& params)
        : duration_ms_{},
          thread_{[this, sql, params]() { this->exec(sql, params); }}
    { }
    const Database::Result& join()
    {
        thread_.join();
        GetDatabaseResult visitor;
        return result_.apply_visitor(visitor);
    }
    void wait() noexcept
    {
        if (thread_.joinable())
        {
            thread_.join();
        }
    }
    int get_duration_in_ms() const
    {
        return duration_ms_;
    }
private:
    template <typename T>
    void solve(T callable)
    {
        auto t0 = std::chrono::steady_clock().now();
        try
        {
            const auto conn = Database::get_default_manager<Database::StandaloneConnectionFactory>().acquire();
            t0 = std::chrono::steady_clock().now();
            result_ = callable(*conn);
        }
        catch (const Database::ResultFailed& e)
        {
            result_ = e;
        }
        catch (const Database::Exception& e)
        {
            result_ = e;
        }
        catch (const std::runtime_error& e)
        {
            result_ = e;
        }
        catch (const std::exception& e)
        {
            result_ = std::runtime_error(e.what());
        }
        catch (...)
        {
            result_ = std::runtime_error("unexpected exception caught");
        }
        const auto t1 = std::chrono::steady_clock().now();
        duration_ms_ = std::chrono::duration_cast<std::chrono::milliseconds>(t1 - t0).count();
        FREDLOG_DEBUG("exec done in " + std::to_string(duration_ms_) + "ms");
    }
    void exec(const std::string& sql)
    {
        this->solve([&](Database::StandaloneConnection& conn) { return conn.exec(sql); });
    }
    void exec(const std::string& sql, const Database::QueryParams& params)
    {
        this->solve([&](Database::StandaloneConnection& conn) { return conn.exec_params(sql, params); });
    }
    using ExecResult = boost::variant<Database::Result,
                                      Database::ResultFailed,
                                      Database::Exception,
                                      std::runtime_error>;
    struct GetDatabaseResult : boost::static_visitor<const Database::Result&>
    {
        const Database::Result& operator()(const Database::Result& result) const
        {
            return result;
        }
        template <typename E>
        const Database::Result& operator()(const E& e) const
        {
            throw e;
        }
    };
    ExecResult result_;
    int duration_ms_;
    std::thread thread_;
};

SqlInThreadSolver::SqlInThreadSolver()
    : running_task_()
{
}

SqlInThreadSolver::SqlInThreadSolver(SqlInThreadSolver&& src)
    : running_task_(std::move(src.running_task_))
{
}

SqlInThreadSolver::SqlInThreadSolver(const std::string& sql)
    : running_task_(std::make_unique<Solver>(sql))
{
}

SqlInThreadSolver::SqlInThreadSolver(const std::string& sql, const Database::QueryParams& params)
    : running_task_(std::make_unique<Solver>(sql, params))
{
}

SqlInThreadSolver::~SqlInThreadSolver()
{
    if (running_task_ != nullptr)
    {
        running_task_->wait();
    }
}

SqlInThreadSolver& SqlInThreadSolver::operator=(SqlInThreadSolver&& src)
{
    std::swap(running_task_, src.running_task_);
    return *this;
}

bool SqlInThreadSolver::is_joinable() const noexcept
{
    return running_task_ != nullptr;
}

SqlInThreadSolver::Result::Result()
    : result_(),
      duration_ms_()
{
}

SqlInThreadSolver::Result::Result(const Result& src)
    : result_(src.result_),
      duration_ms_(src.duration_ms_)
{
}

SqlInThreadSolver::Result::Result(SqlInThreadSolver&& solver)
    : result_(solver.running_task_->join()),
      duration_ms_(solver.running_task_->get_duration_in_ms())
{
    solver.running_task_.reset(nullptr);
}

SqlInThreadSolver::Result::~Result()
{
}

SqlInThreadSolver::Result& SqlInThreadSolver::Result::operator=(const Result& src)
{
    result_ = src.result_;
    duration_ms_ = src.duration_ms_;
    return *this;
}

SqlInThreadSolver::Result& SqlInThreadSolver::Result::operator=(SqlInThreadSolver&& solver)
{
    result_ = solver.running_task_->join();
    duration_ms_ = solver.running_task_->get_duration_in_ms();
    solver.running_task_.reset(nullptr);
    return *this;
}

const Database::Result& SqlInThreadSolver::Result::get_result() const
{
    return result_;
}

int SqlInThreadSolver::Result::get_duration_in_ms() const
{
    return duration_ms_;
}

}//namespace Fred::Registry::Util
}//namespace Fred::Registry
}//namespace Fred
